"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.series = exports.parallel = exports.newScript = exports.noop = exports.isFunction = exports.isDefined = void 0;

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var isDefined = function isDefined(val) {
  return val != null;
};

exports.isDefined = isDefined;

var isFunction = function isFunction(val) {
  return typeof val === 'function';
};

exports.isFunction = isFunction;

var noop = function noop(_) {};

exports.noop = noop;

var newScript = function newScript(src) {
  return function (cb) {
    var scriptElem = document.createElement('script');

    if (_typeof(src) === 'object') {
      // copy every property to the element
      for (var key in src) {
        if (Object.prototype.hasOwnProperty.call(src, key)) {
          scriptElem[key] = src[key];
        }
      }

      src = src.src;
    } else {
      scriptElem.src = src;
    }

    scriptElem.addEventListener('load', function () {
      return cb(null, src);
    });
    scriptElem.addEventListener('error', function () {
      return cb(true, src);
    });
    document.body.appendChild(scriptElem);
    return scriptElem;
  };
};

exports.newScript = newScript;

var keyIterator = function keyIterator(cols) {
  var keys = Object.keys(cols);
  var i = -1;
  return {
    next: function next() {
      i++; // inc

      if (i >= keys.length) return null;else return keys[i];
    }
  };
}; // tasks should be a collection of thunk


var parallel = function parallel() {
  for (var _len = arguments.length, tasks = new Array(_len), _key = 0; _key < _len; _key++) {
    tasks[_key] = arguments[_key];
  }

  return function (each) {
    return function (cb) {
      var hasError = false;
      var successed = 0;
      var ret = [];
      tasks = tasks.filter(isFunction);
      if (tasks.length <= 0) cb(null);else {
        tasks.forEach(function (task, i) {
          var thunk = task;
          thunk(function (err) {
            for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
              args[_key2 - 1] = arguments[_key2];
            }

            if (err) hasError = true;else {
              // collect result
              if (args.length <= 1) args = args[0];
              ret[i] = args;
              successed++;
            }
            if (isFunction(each)) each.call(null, err, args, i);
            if (hasError) cb(true);else if (tasks.length === successed) {
              cb(null, ret);
            }
          });
        });
      }
    };
  };
}; // tasks should be a collection of thunk


exports.parallel = parallel;

var series = function series() {
  for (var _len3 = arguments.length, tasks = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    tasks[_key3] = arguments[_key3];
  }

  return function (each) {
    return function (cb) {
      tasks = tasks.filter(function (val) {
        return val != null;
      });
      var nextKey = keyIterator(tasks);

      var nextThunk = function nextThunk() {
        var key = nextKey.next();
        var thunk = tasks[key];
        if (Array.isArray(thunk)) thunk = parallel.apply(null, thunk).call(null, each);
        return [+key, thunk]; // convert `key` to number
      };

      var key, thunk;
      var next = nextThunk();
      key = next[0];
      thunk = next[1];
      if (thunk == null) return cb(null);
      var ret = [];

      var iterator = function iterator() {
        thunk(function (err) {
          for (var _len4 = arguments.length, args = new Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
            args[_key4 - 1] = arguments[_key4];
          }

          if (args.length <= 1) args = args[0];
          if (isFunction(each)) each.call(null, err, args, key);
          if (err) cb(err);else {
            // collect result
            ret.push(args);
            next = nextThunk();
            key = next[0];
            thunk = next[1];
            if (thunk == null) return cb(null, ret); // finished
            else iterator();
          }
        });
      };

      iterator();
    };
  };
};

exports.series = series;